use std::{borrow::Cow, string::FromUtf8Error};

use crate::utils::urlencoded::endecode::EnCodable;

use super::endecode::DeCodable;

#[derive(Clone)]
pub struct UrlEncodeData {
    encoded: String,
    raw: Vec<u8>,
    raw_string: Option<String>,
}

impl UrlEncodeData {
    pub fn from_raw<T: AsRef<[u8]>>(raw: T) -> Result<Self, FromUtf8Error> {
        Ok(Self {
            raw: raw.as_ref().to_owned(),
            encoded: raw.as_ref().encode(),
            raw_string: String::from_utf8(raw.as_ref().into()).ok(),
        })
    }
    pub fn from_encoded(encoded: &str) -> Result<Self, ()> {
        Ok(Self {
            encoded: encoded.to_owned(),
            raw: encoded.decode()?,
            raw_string: String::from_utf8(encoded.decode()?).ok(),
        })
    }

    pub fn encoded(&self) -> &str {
        self.encoded.as_ref()
    }
    pub fn raw(&self) -> &[u8] {
        self.raw.as_ref()
    }
    pub fn raw_string(&self) -> Option<&str> {
        self.raw_string.as_ref().map(|x| x.as_str())
    }
}

impl std::fmt::Display for UrlEncodeData {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{}", self.encoded)
    }
}
